;:::::::: FONCTIONS RELATIVES AU CENTRE DE GRAVITE ::::::::

; Calcul du centre de gravité d'un édifice
(defun cdg (LE)
  (list
   (/ (float (apply '+ (mapcar '(lambda (cube) (accesg cube 'x)) LE))) (float (length LE)))
   (/ (float (apply '+ (mapcar '(lambda (cube) (accesg cube 'y)) LE))) (float (length LE)))))

; Construire une équation avec deux points
(defun equation (pt1 pt2)
  (list '- (list '* (list '- 'x (car pt1)) (- (cadr pt2) (cadr pt1))) (list '* (list '- 'y (cadr pt1)) (- (car pt2) (car pt1)))))

; Construire toutes les équations de droites possible aves une liste de coordonnées x, y
(defun equations (LPT LQ)
  (cond ((null (cdr LPT))  LQ)
	(t                 (equations (cdr LPT) (append (mapcar '(lambda (pt) (equation pt (car LPT))) (cdr LPT)) LQ))   )))

; Evaluer une équation avec les coordonnées d'un point
(defun evalequa (e pt)
  (cond ((eq (aieme-terme '(1 1 1) e) 'x)
	 (eval (chaieme '(1 1 1) (car pt) (chaieme '(2 1 1) (cadr pt) e))))
	((eq (aieme-terme '(1 1 1) e) 'y)
	 (eval (chaieme '(1 1 1) (cadr pt) (chaieme '(2 1 1) (car pt) e))))))

; Fonction qui évalue une équation pour une liste de points
(defun evalequalpt (e LPT) 
  (mapcar '(lambda (CPT) (evalequa e CPT)) LPT))

; Fonction qui évalue une liste d'équations avec un point
(defun evalequaspt (LQ pt)
  (mapcar '(lambda (e) (evalequa e pt)) LQ))

; Fonction qui test si une liste de nombres est entièrement positif ou nul
(defun positif (Lnb)
  (every '(lambda (nb) (>= nb 0)) Lnb))

; Fonction qui test si une liste de nombres est entièrement négatif ou nul
(defun negatif (Lnb)
  (every '(lambda (nb) (<= nb 0)) Lnb))

; Fonction qui rends l'opposé d'une équation
(defun oppequa (e)
  (cons '- (list (nth '2 e) (nth '1 e))))

; IL FAUT SELECTIONNER LES BONNES DROITES
; Sélectionner les équations "extérieures" (le polygone de sustentation est convexe)
(defun equationsext (LPT LQ LR)
  (cond ((null LQ)                             LR)
	((positif (evalequalpt (car LQ) LPT)) (equationsext LPT (cdr LQ) (cons (car LQ) LR)))
	((negatif (evalequalpt (car LQ) LPT)) (equationsext LPT (cdr LQ) (cons (oppequa (car LQ)) LR)))
	(t                                    (equationsext LPT (cdr LQ) LR))))

; Fonction qui prends un point et renvoie les quatre points aux "coins"
(defun coins (pt)
  (mapcar '(lambda (cpl) (list (+ (car cpl) (car pt)) (+ (cadr cpl) (cadr pt)))) '((1 1) (-1 1) (1 -1) (-1 -1))))

; Dans les cas particulier où on a seulement un ou deux cubes au sol,
; on élargit légèrement le polygone de sustantation en ajoutant les points des
; coins des cubes à la liste
(defun cas1ou2 (LPT)
  (cond ((= (length LPT) 1) (coins (car LPT)))
	((= (length LPT) 2) (append (coins (car LPT)) (coins (cadr LPT))))
	(t                  LPT)))

; Combiner equationsext et equations (pour avoir 1 seul appel à edifice)
(defun polysust (LPT)
  (equationsext LPT (equations LPT 'nil) 'nil))

; Tester la stabilité d'un edifice
(defun stable (LE)
  (positif (evalequaspt (polysust (cas1ou2 (listerxy (listerbase LE)))) (cdg LE))))