;::::::::: RECHERCHE DE CHEMIN ::::::::

; Recherche d'un chemin
; cubea : cube de départ
; cubeb : cube d'arrivée
(defun chemin (cubea cubeb)
  (algoa cubea (majouvert (listeouv cubea) nil cubeb 0) (list (list nil cubea)) cubeb))

; Algorithme A*
; depart : le cube de départ (qui change récursivement)
; LO : la liste "ouvert" -> les voies non développés
; LA : l'"arbre" du chemin parcouru
; but : le cube d'arrivée (qui ne change jamais)
(defun algoa (depart LO LA but)
  (cond ((eq depart but) (append (cdr (reconstituer LA but nil)) (list but)))
	(t               (algoa (caar LO)
				(majouvert (retire2 depart (listeouv (caar LO))) (cdr LO) but (cadr (car LO)))
				(cons (cons depart (listeouv depart)) LA)
				but))))

; nb :
; Un élément de "ouvert" :
; (cube cpt heuristique)
; cpt = (cadr touv)
; heuristique = (cadr (cdr touv))

; Fonction heuristique : estimation du chemin restant
; On prends ici la  distance entre deux cubes
(defun heuristique (cubea cubeb)
  (sqrt (+ (carre (- (accesg cubeb 'x) (accesg cubea 'x)))
	   (carre (- (accesg cubeb 'y) (accesg cubea 'y)))
	   (carre (- (accesg cubeb 'z) (accesg cubea 'z))))))

; Renvoyer toutes les "issuess" possibles d'un cube (liste de cubes)
(defun listeouv (cube)
  (nettoievide (mapcar '(lambda (face)
			  (cond ((eq (accesg cube face) 1) (if (voisouv cube face) (accesg cube (fasansf face)) 'vide))
				(t                         'vide)))
		       '(fnord fsud fouest fest fhaut fbas))))
; ATTENTION! nettoievide est dans arc.majcubes.lisp

; Teste si le voisin du côté "face" du cube a une ouverture
(defun voisouv (cube face)
  (if (eq (accesg cube (fasansf face)) 'vide)
      nil
      (if (eq (accesg
	       (accesg cube (fasansf face))
	       (cadr (assoc face '((fnord fsud) (fsud fnord) (fouest fest) (fest fouest) (fhaut fbas) (fbas fhaut)))))
	      1)
	  t
	  nil)))

; Ajout d'une liste de cubes à la liste "ouvert" et mise à jour
; - Niveau dans l'arbre du nouveau cube (cpt)
; - Calcul de l'heuristique pour chaque nouveau cube
; - Tri de la liste créée par ordre de cpt + heuristique croissant
(defun majouvert (LC LO but cpt)
  (cond ((null LC) LO)
	(t         (majouvert (cdr LC)
			      (trier (list (car LC) (1+ cpt) (heuristique (car LC) but)) LO nil)
			      but
			      cpt))))

; Trier un nouvel élément de "ouvert" (selon le chemin espéré croissant)
(defun trier (elm LO LOt)
  (cond ((null LO)                                                                      (append LOt (list elm)))
	((<= (+ (cadr (cdr elm)) (cadr elm)) (+ (cadr (cdr (car LO))) (cadr (car LO)))) (append LOt (cons elm LO)))
	(t                                                                         (trier elm (cdr LO) (append LOt (list (car LO)))))))

; Reconstituer le chemin parcouru (en effaçant les branches mortes)
; Remplacer la deuxième condition par (app svg (cdr (car arbre))) tout court
; pour éviter certaines erreurs de reconstituer (et en créer d'autres)
; --> la vraie solution : un ID pour chque étape
; bkp : (and (app svg (cdr (car arbre))) (not (app (caar arbre) L)))
(defun reconstituer (arbre svg L)
  (cond ((null arbre)                L)
	((app svg (cdr (car arbre))) (reconstituer (cdr arbre) (caar arbre) (cons (caar arbre) L)))
	(t                           (reconstituer (cdr arbre) svg L))))

; Fonctions utiles
; fface -> face
(defun fasansf (face)
  (cadr (assoc face '((fnord nord) (fsud sud) (fouest ouest) (fest est) (fhaut haut) (fbas bas)))))
; face -> fface
(defun sansfaf (face)
  (cadr (assoc face '((nord fnord) (sud fsud) (ouest fouest) (est fest) (haut fhaut) (bas fbas)))))
; Dernier élément d'une liste
;(defun dernier (L)
;  (cond ((null (cdr L)) (car L))
;	(t              (dernier (cdr L)))))

